require "spec_helper"

describe Micropub::Token do
  let(:token) { Micropub::Token.new("1234") }

  describe ".initialize" do
    it "requires a string token" do
      expect { Micropub::Token.new }.to raise_error ArgumentError
    end
  end

  describe "making requests" do
    let(:endpoint) { Micropub.configuration.token_endpoint }
    let(:raw_response) { "me=https://person.com/&client_id=https://ownyourgram.com&scope=post,patch&issued_at=1399155608&nonce=501884823" }
    before do
      stub_request(:get, endpoint).with(headers: {"Authorization" => "Bearer 1234"}).to_return(body: raw_response)
    end

    describe "#valid?" do
      context "with valid params" do
        it "accepts multiple scopes" do
          expect(token.valid?).to eq true
        end
      end
      context "fails with invalid params" do
        it "with wrong scope" do
          Micropub.configure {|c| c.allowed_scopes = [:delete] }
          expect(token.valid?).to eq false
        end

        it "with wrong host" do
          Micropub.configure {|c| c.me = "http://blah.com" }
          expect(token.valid?).to eq false
        end
      end

      context "when no me in token server" do
        let(:raw_response) { "client_id=https://ownyourgram.com&scope=post,patch&issued_at=1399155608&nonce=501884823" }
        it "returns false" do
          expect(token.valid?).to eq false
        end
      end
    end

    describe "#me" do
      let(:raw_response) { "client_id=https://ownyourgram.com&scope=post,patch&issued_at=1399155608&nonce=501884823" }
      it "raises an error when token server me is not present" do
        expect { token.me }.to raise_error Micropub::ValidationError
      end
    end

    describe "#response" do
      it "returns a hash" do
        response = {
          "me"        => "https://person.com/",
          "client_id" => "https://ownyourgram.com",
          "scope"     => "post,patch",
          "issued_at" => "1399155608",
          "nonce"     => "501884823"}
        expect(token.response).to eq(response)
      end
    end
  end
end
